<?php
/**
 * Kalium WordPress Theme
 *
 * Theme header functions.
 *
 * @author Laborator
 * @link   https://kaliumtheme.com
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Direct access not allowed.
}

/**
 * Show or hide header.
 *
 * @return bool
 */
function kalium_show_header() {
	return apply_filters( 'kalium_show_header', true );
}

/**
 * Check if the header type is custom header.
 *
 * @return bool
 */
function kalium_is_custom_header() {
	return 'custom-header' === kalium_get_theme_option( 'main_menu_type' );
}

/**
 * Check if header is fullwidth.
 *
 * @return bool
 */
function kalium_is_fullwidth_header() {
	$return = boolval( kalium_get_theme_option( 'header_fullwidth' ) );

	if ( $queried_object_id = kalium_get_queried_object_id() ) {
		$header_fullwidth = kalium_get_field( 'header_fullwidth', $queried_object_id );

		if ( in_array( $header_fullwidth, [ 'yes', 'no' ] ) ) {
			$return = 'yes' === $header_fullwidth;
		}
	}

	return $return;
}

/**
 * Check if header contains top or side menu.
 *
 * @param string{'top-menu'|'sidebar-menu'} $menu_type
 *
 * @return bool
 */
function kalium_header_has_content_element_type( $menu_type ) {
	static $header_content_elements;

	// Custom Header Content Entries
	if ( ! isset( $header_content_elements ) ) {
		$header_content_elements = array_map( 'kalium_parse_content_builder_field', [
			kalium_get_theme_option( 'custom_header_content_left' ),
			kalium_get_theme_option( 'custom_header_content_right' ),
			kalium_get_theme_option( 'custom_header_content' ),
		] );
	}

	foreach ( $header_content_elements as $menu_content_entry ) {
		if ( $entries = kalium_get_array_key( $menu_content_entry, 'entries' ) ) {
			foreach ( $entries as $entry ) {
				if ( 'element' === $entry['content']['type'] ) {

					// Check for "top menu"
					if ( 'top-menu' === $menu_type && 'open-top-menu' === $entry['content']['value'] ) {
						return true;
					} // Check for "sidebar menu"
					else if ( 'sidebar-menu' === $menu_type && 'open-sidebar-menu' === $entry['content']['value'] ) {
						return true;
					}
				}
			}
		}
	}

	return false;
}

/**
 * Check if header has top bar menu.
 *
 * @return bool
 */
function kalium_header_has_top_bar() {
	return boolval( kalium_get_theme_option( 'top_header_bar' ) );
}

/**
 * Parse content builder field.
 *
 * @param string $str
 *
 * @return array
 */
function kalium_parse_content_builder_field( $str ) {

	// Parsed JSON
	$parsed = json_decode( $str, true );

	// Empty JSON to array
	if ( ! is_array( $parsed ) ) {
		$parsed = [];
	}

	// Entries list
	$entries = kalium_get_array_key( $parsed, 'entries', [] );

	// Map to properly format entries
	$entries = array_map( 'kalium_parse_content_builder_entry', $entries );

	// Options array
	$options = [

		// Alignment
		'alignment' => kalium_get_array_key( $parsed, 'alignment' ),
	];

	return [
		'options' => $options,
		'entries' => $entries,
	];
}

/**
 * Parse menu content element type and return the modified array.
 *
 * @param array $entry
 *
 * @return array
 */
function kalium_parse_content_builder_entry( $entry ) {

	// When there is no content type defined
	if ( ! isset( $entry['contentType'] ) ) {
		return $entry;
	}

	// Nav menus
	$registered_nav_menus = get_registered_nav_menus();

	// Content type
	$content_type = $entry['contentType'];

	// Content category type and value
	$content = [
		'type'  => 'element',
		'value' => $content_type,
	];

	// Parse menu entries
	if ( preg_match( '/^menu-([0-9]+)$/', $content_type, $matches ) ) {
		$content['type']  = 'menu';
		$content['value'] = $matches[1];
	} // Parse registered nav menu location
	else if ( isset( $registered_nav_menus[ $content_type ] ) ) {
		$content['type']  = 'menu';
		$content['value'] = $content_type;
	}

	// Hide on devices
	if ( isset( $entry['options']['hide_on'] ) ) {
		$hide_on         = $entry['options']['hide_on'];
		$hide_on_desktop = in_array( 'desktop', $hide_on );
		$hide_on_tablet  = in_array( 'tablet', $hide_on );
		$hide_on_mobile  = in_array( 'mobile', $hide_on );

		$entry['options']['hide_on'] = [
			'desktop' => $hide_on_desktop,
			'tablet'  => $hide_on_tablet,
			'mobile'  => $hide_on_mobile,
		];
	}

	return wp_parse_args( $entry, [
		'contentType' => '',
		'content'     => [
			'type'  => $content['type'],
			'value' => $content['value'],
		],
	] );
}

/**
 * Get header option.
 *
 * @param string $option
 *
 * @return mixed|null
 */
function kalium_header_get_option( $option ) {
	$return            = null;
	$queried_object_id = kalium_get_queried_object_id();

	switch ( $option ) {

		// Position
		case 'position':
			$theme_option_field = $post_meta_field = 'header_position';
			break;

		// Spacing
		case 'spacing':
			$theme_option_field = $post_meta_field = 'header_spacing';

			// Ignore post meta field when header position is not defined in post meta
			if ( 'inherit' === kalium_get_field( 'header_position', $queried_object_id ) ) {
				$post_meta_field = null;
			}
			break;

		// Default
		default:
			return null;
	}

	// Get value from theme options
	$return = kalium_get_theme_option( $theme_option_field );

	// Get value from page settings
	if ( $queried_object_id > 0 && $post_meta_field ) {
		$page_value = kalium_get_field( $post_meta_field, $queried_object_id );

		if ( 'inherit' !== $page_value ) {
			$return = $page_value;
		}
	}

	return $return;
}

/**
 * New implementation of Sticky Header.
 *
 * @return array
 */
function kalium_get_sticky_header_options() {

	// Options
	$options = [

		// Sticky menu type: standard, autohide
		'type'                      => 'standard',

		// Sticky container selector
		'containerElement'          => '.site-header',

		// Logo selector
		'logoElement'               => '.logo-image',

		// Trigger hook offset relative to screen
		'triggerOffset'             => 0,

		// Initial offset for all animation scenes
		'offset'                    => 0,

		// Initial offset for animation scene (sticky type: standard)
		'animationOffset'           => 10,

		// Spacer to cover up the header height
		'spacer'                    => true,

		// Animate duration with scroll
		'animateProgressWithScroll' => true,

		// Animate duration (if not null, overwrites animation duration on timeline)
		'animateDuration'           => null,

		// Tween changes for animation scene
		'tweenChanges'              => false,

		// Sticky header class name definitions
		'classes'                   => [

			// Element name
			'name'     => 'site-header',

			// Class prefix for sticky header options
			'prefix'   => 'sticky',

			// Initialized class
			'init'     => 'initialized',

			// Menu is set to fixed
			'fixed'    => 'fixed',

			// Menu is set to absolute
			'absolute' => 'absolute',

			// Sticky header spacer class
			'spacer'   => 'spacer',

			// Sticky header is active
			'active'   => 'active',
		],

		// Autohide options
		'autohide'                  => [

			// Animation type
			'animationType' => 'fade-slide-top',

			// Animate duration
			'duration'      => 0.3,

			// Threshold distance before menu is shown/hide
			'threshold'     => 100,
		],

		// Props to animate
		'animateScenes'             => [],

		// Alternate logos
		'alternateLogos'            => [

			// Sticky logo
			//	'sticky'   => [
			//		'name'  => 'sticky',
			//		'image' => wp_get_attachment_image( 1, 'original' ),
			//		'css'   => [
			//			'width'  => 30,
			//			'height' => 30,
			//		],
			//	],
		],

		// Supported on devices
		'supportedOn'               => [

			// Desktop
			'desktop' => true,

			// Tablet
			'tablet'  => false,

			// Mobile
			'mobile'  => true,
		],

		// Other options
		'other'                     => [

			// Menu skin on sticky mode
			'menuSkin' => null,
		],

		// Debug mode
		'debugMode'                 => defined( 'KALIUM_DEBUG' ) && KALIUM_DEBUG,
	];

	// Animate scenes
	$animate_scenes = [];

	// Padding animation
	//	$animate_scenes['padding'] = [
	//		'name'     => 'padding',
	//		'selector' => 'self',
	//		'props'    => [
	//			'paddingTop',
	//			'paddingBottom',
	//		],
	//		'css'      => [
	//			'default' => [
	//				'paddingTop'    => 10,
	//				'paddingBottom' => 10,
	//			],
	//		],
	//	];

	// Background animation
	//	$animate_scenes['background'] = [
	//		'name'     => 'background',
	//		'selector' => 'self',
	//		'props'    => [
	//			'backgroundColor',
	//		],
	//		'css'      => [
	//			'default' => [
	//				'backgroundColor' => '#c0c0c0',
	//			],
	//		],
	//	];

	// Resize logo animation
	//	$animate_scenes['resizeLogo'] = [
	//		'name'     => 'logo-resize',
	//		'selector' => '.logo-image',
	//		'props'    => [
	//			'width',
	//			'height',
	//		],
	//		'css'      => [
	//			'default' => [
	//				'width'  => 30,
	//				'height' => 30,
	//			],
	//		],
	//	];

	// Sticky logo animation
	//	$animate_scenes['stickyLogo'] = [
	//		'name'     => 'sticky-logo',
	//		'selector' => '.logo-image',
	//		'props'    => [
	//			'width',
	//			'height',
	//		],
	//		'data'     => [
	//			'type'          => 'alternate-logo',
	//			'alternateLogo' => 'sticky',
	//		],
	//		'duration' => 0.6,
	//		'css'      => [
	//			'width'  => 40,
	//			'height' => 30,
	//		],
	//	];

	// Add animate scene array
	//	$options['animateScenes'] = $animate_scenes;

	/**
	 * Hook: kalium_sticky_header_options.
	 *
	 * @hooked kalium_sticky_header_options_default - 10
	 * @hooked kalium_sticky_options_for_static_header_type - 20
	 */
	return apply_filters( 'kalium_sticky_header_options', $options );
}

/**
 * Get logo switch sections.
 *
 * @return array
 */
function kalium_get_logo_switch_sections() {
	$sections = [];

	if ( is_singular() && kalium_get_field( 'section_logo_switch' ) ) {
		$sections = kalium_get_field( 'logo_switch_sections' );
	}

	return apply_filters( 'kalium_sticky_logo_switch_sections', $sections );
}
