<?php
/**
 * Kalium WordPress Theme
 *
 * Header hook functions.
 *
 * @author Laborator
 * @link   https://kaliumtheme.com
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Direct access not allowed.
}

/**
 * Default sticky options.
 *
 * @param array $options
 *
 * @return array
 */
function _kalium_sticky_header_options_default( $options ) {

	// Sticky vars
	$vertical_padding    = kalium_get_theme_option( 'sticky_header_vertical_padding' );
	$background_color    = kalium_get_theme_option( 'sticky_header_background_color' );
	$header_skin         = kalium_get_theme_option( 'sticky_header_skin' );
	$has_border          = kalium_get_theme_option( 'sticky_header_border' );
	$border_color        = kalium_get_theme_option( 'sticky_header_border_color' );
	$border_width        = kalium_get_theme_option( 'sticky_header_border_width' );
	$shadow_color        = kalium_get_theme_option( 'sticky_header_shadow_color' );
	$shadow_width        = kalium_get_theme_option( 'sticky_header_shadow_width' );
	$shadow_blur         = kalium_get_theme_option( 'sticky_header_shadow_blur' );
	$is_autohide         = kalium_get_theme_option( 'sticky_header_autohide' );
	$autohide_type       = kalium_get_theme_option( 'sticky_header_autohide_animation_type' );
	$autohide_duration   = kalium_get_theme_option( 'sticky_header_autohide_duration' );
	$logo_width          = kalium_get_theme_option( 'sticky_header_logo_width' );
	$custom_logo         = kalium_get_theme_option( 'sticky_header_logo' );
	$animate_with_scroll = kalium_get_theme_option( 'sticky_header_animate_duration' );
	$tween_changes       = kalium_get_theme_option( 'sticky_header_tween_changes' );
	$animate_duration    = kalium_get_theme_option( 'header_sticky_duration' );
	$animate_offset      = kalium_get_theme_option( 'header_sticky_initial_offset' );
	$animate_chaining    = kalium_get_theme_option( 'sticky_header_animation_chaining' );
	$supported_on        = [
		'desktop' => kalium_get_theme_option( 'sticky_header_support_desktop' ),
		'tablet'  => kalium_get_theme_option( 'sticky_header_support_tablet' ),
		'mobile'  => kalium_get_theme_option( 'sticky_header_support_mobile' ),
	];

	/**
	 * Initial offset for scenes to exclude top header bar.
	 */
	$options['offset'] = '.top-header-bar';

	/**
	 * Animation scene offset.
	 */
	if ( is_numeric( $animate_offset ) ) {
		$options['animationOffset'] = $animate_offset;
	}

	/**
	 * Animate animation progress with scroll.
	 */
	$options['animateProgressWithScroll'] = ! boolval( $animate_with_scroll );

	/**
	 * Tween changes with scroll progress.
	 */
	$options['tweenChanges'] = boolval( $tween_changes );

	/**
	 * Animate duration in seconds.
	 */
	if ( is_numeric( $animate_duration ) ) {
		$options['animateDuration'] = $animate_duration;
	}

	/**
	 * Generate spacer or not.
	 */
	$options['spacer'] = 'absolute' !== kalium_header_get_option( 'position' );

	/**
	 * Autohide sticky type.
	 */
	if ( $is_autohide ) {

		// Sticky type
		$options['type'] = 'autohide';

		// Animation type
		$options['autohide']['animationType'] = $autohide_type;

		// Duration
		if ( is_numeric( $autohide_duration ) ) {
			$options['autohide']['duration'] = $autohide_duration;
		}
	}

	/**
	 * Supported on device types.
	 */
	$options['supportedOn'] = $supported_on;

	/**
	 * Menu skin on sticky mode.
	 */
	if ( $header_skin ) {
		$options['other']['menuSkin'] = $header_skin;
	}

	/**
	 * Animation scenes.
	 */
	$scene_name_padding     = 'padding';
	$scene_name_styling     = 'styling';
	$scene_name_sticky_logo = 'sticky-logo';
	$scene_name_resize_logo = 'resize-logo';

	$animation_scenes = [];

	// Padding animation scene
	if ( is_numeric( $vertical_padding ) ) {

		$animation_scene_padding = [
			'name'     => 'padding',
			'selector' => '.header-block',
			'props'    => [
				'paddingTop',
				'paddingBottom',
			],
			'css'      => [
				'default' => [
					'paddingTop'    => $vertical_padding,
					'paddingBottom' => $vertical_padding,
				],
			],
		];

		$animation_scenes[ $scene_name_padding ] = $animation_scene_padding;
	}

	// Background, shadow and border animation scene
	$animation_scene_style = [
		'name'     => 'style',
		'selector' => '.header-block',
		'props'    => [],
		'css'      => [
			'default' => [],
		],
		'data'     => [
			'tags' => [
				'transparent-header', // required!
			],
		],
	];

	if ( $background_color ) {
		$animation_scene_style['props'][]                           = 'backgroundColor';
		$animation_scene_style['css']['default']['backgroundColor'] = $background_color;
	}

	// Bottom border
	if ( $has_border ) {

		// Bottom border color
		if ( $border_color ) {
			$animation_scene_style['props'][]                        = 'borderBottom';
			$animation_scene_style['css']['default']['borderBottom'] = sprintf( '%s solid %s', $border_width, $border_color );
			kalium_append_custom_css( '.site-header', 'border-bottom: 1px solid transparent' );
		}

		// Shadow color
		if ( $shadow_color ) {
			$animation_scene_style['props'][]                     = 'boxShadow';
			$animation_scene_style['css']['default']['boxShadow'] = sprintf( '%s 0px %s %s', $shadow_color, $shadow_width, $shadow_blur );
			kalium_append_custom_css( '.site-header', 'box-shadow: 0px 0px 0px transparent' );
		}
	}

	// Register styling scene if there are props to animate
	if ( ! empty( $animation_scene_style['css']['default'] ) ) {
		$animation_scenes[ $scene_name_styling ] = $animation_scene_style;
	}

	// Custom logo animation scene
	if ( $custom_logo ) {

		// Attachment image
		$sticky_logo = wp_get_attachment_image( $custom_logo, 'original' );

		if ( $sticky_logo ) {

			// Register alternate logo
			$sticky_logo_src    = wp_get_attachment_image_src( $custom_logo, 'original' );
			$sticky_logo_width  = $sticky_logo_src[1];
			$sticky_logo_height = $sticky_logo_src[2];

			$options['alternateLogos']['sticky'] = [
				'name'  => 'sticky',
				'image' => $sticky_logo,
			];

			// Custom logo width
			if ( $logo_width ) {
				$new_dimensions = kalium()->helpers->resize_by_width( $sticky_logo_width, $sticky_logo_height, $logo_width );

				$sticky_logo_width  = $new_dimensions[0];
				$sticky_logo_height = $new_dimensions[1];
			} else {

				// Determine logo width set in theme options
				$custom_logo_max_width = kalium_get_theme_option( 'custom_logo_max_width' );
				$new_dimensions        = kalium()->helpers->resize_by_width( $sticky_logo_width, $sticky_logo_height, $custom_logo_max_width );

				$sticky_logo_width  = $new_dimensions[0];
				$sticky_logo_height = $new_dimensions[1];
			}

			// Sticky logo animation scene
			$animation_scene_sticky_logo = [
				'name'     => 'sticky-logo',
				'selector' => 'logo',
				'props'    => [
					'width',
					'height',
				],
				'css'      => [
					'width'  => $sticky_logo_width,
					'height' => $sticky_logo_height,
				],
				'data'     => [
					'type'          => 'alternate-logo',
					'alternateLogo' => 'sticky',
					'tags'          => [
						'logo-switch', // required!
					],
				],
			];

			$animation_scenes[ $scene_name_sticky_logo ] = $animation_scene_sticky_logo;
		}
	} // Logo resize animation scene
	else if ( $logo_width && ( $custom_logo = kalium_get_theme_option( 'custom_logo_image' ) ) ) {
		$custom_logo_src = wp_get_attachment_image_src( $custom_logo, 'original' );

		if ( $custom_logo_src ) {
			$custom_logo_width  = $custom_logo_src[1];
			$custom_logo_height = $custom_logo_src[2];
			$new_dimensions     = kalium()->helpers->resize_by_width( $custom_logo_width, $custom_logo_height, $logo_width );

			$custom_logo_width  = $new_dimensions[0];
			$custom_logo_height = $new_dimensions[1];

			// Resize logo animation scene
			$animation_scenes[ $scene_name_resize_logo ] = [
				'name'     => 'logo-resize',
				'selector' => '.logo-image',
				'props'    => [
					'width',
					'height',
				],
				'css'      => [
					'default' => [
						'width'  => $custom_logo_width,
						'height' => $custom_logo_height,
					],
				],
				'data'     => [
					'tags' => [
						'logo-resize', // required!
					],
				],
			];
		}
	}

	/**
	 * Animation chaining.
	 */
	$scene_duration_base = 0.3;

	switch ( $animate_chaining ) {

		// All at once
		case 'all':
			foreach ( $animation_scenes as & $animation_scene ) {
				$animation_scene['position'] = 0;
			}
			break;

		// Padding -> Background, Logo
		case 'padding-bg_logo':
			$animation_chain = [
				$scene_name_padding,
				[ $scene_name_styling, $scene_name_sticky_logo, $scene_name_resize_logo ],
			];
			break;

		// Background, Logo -> Padding
		case 'bg_logo-padding':
			$animation_chain = [
				[ $scene_name_styling, $scene_name_sticky_logo, $scene_name_resize_logo ],
				$scene_name_padding,
			];
			break;

		// Logo, Padding -> Background
		case 'logo_padding-bg':
			$animation_chain = [
				[ $scene_name_padding, $scene_name_sticky_logo, $scene_name_resize_logo ],
				$scene_name_styling,
			];
			break;

		// Background -> Logo, Padding
		case 'bg-logo_padding':
			$animation_chain = [
				$scene_name_styling,
				[ $scene_name_padding, $scene_name_sticky_logo, $scene_name_resize_logo ],
			];
			break;

		// Padding -> Background -> Logo
		case 'padding-bg-logo':
			$animation_chain = [
				$scene_name_padding,
				$scene_name_styling,
				[ $scene_name_sticky_logo, $scene_name_resize_logo ],
			];
			break;

		// Background -> Logo -> Padding
		case 'bg-logo-padding':
			$animation_chain = [
				$scene_name_styling,
				[ $scene_name_sticky_logo, $scene_name_resize_logo ],
				$scene_name_padding,
			];
			break;

		// Logo -> Background -> Padding
		case 'logo-bg-padding':
			$animation_chain = [
				[ $scene_name_sticky_logo, $scene_name_resize_logo ],
				$scene_name_styling,
				$scene_name_padding,
			];
			break;

		// Background -> Padding -> Logo
		case 'bg-padding-logo':
			$animation_chain = [
				$scene_name_styling,
				$scene_name_padding,
				[ $scene_name_sticky_logo, $scene_name_resize_logo ],
			];
			break;
	}

	// Set animation chaining
	if ( isset( $animation_chain ) ) {

		foreach ( $animation_chain as $i => $scenes ) {

			$scenes = ! is_array( $scenes ) ? [ $scenes ] : $scenes;

			foreach ( $scenes as $scene_name ) {

				if ( isset( $animation_scenes[ $scene_name ] ) ) {
					$position = $i * $scene_duration_base;

					// Duration on timeline
					if ( ! isset( $animation_scenes[ $scene_name ]['duration'] ) ) {
						$animation_scenes[ $scene_name ]['duration'] = $scene_duration_base;
					}

					// Position on timeline
					$animation_scenes[ $scene_name ]['position'] = $position;
				}
			}
		}
	}

	// Sort animation scenes by their position
	uasort( $animation_scenes, function ( $a, $b ) {
		if ( isset( $a['position'] ) && isset( $b['position'] ) ) {
			return $a['position'] < $b['position'] ? - 1 : 1;
		}
	} );

	// Update animation scenes property
	$options['animateScenes'] = $animation_scenes;

	return $options;
}

/**
 * Sticky options for static header types.
 *
 * @param array $options
 *
 * @return array
 */
function _kalium_sticky_options_for_static_header_type( $options ) {

	// Static header type only
	if ( false === kalium_is_custom_header() ) {
		foreach ( $options['animateScenes'] as $scene_id => &$scene_args ) {
			if ( '.header-block' === $scene_args['selector'] ) {
				$scene_args['selector'] = 'self';
			}
		}
	}

	return $options;
}

/**
 * Header position and spacing.
 *
 * @return void
 */
function _kalium_header_position_spacing_action() {

	$queried_object_id = kalium_get_queried_object_id();

	$header_position = kalium_header_get_option( 'position' );
	$header_spacing  = kalium_header_get_option( 'spacing' );

	// Header position
	if ( 'absolute' === $header_position && ! post_password_required( $queried_object_id ) ) {
		kalium_append_custom_css( '.site-header', 'position: absolute; left: 0; right: 0;' );

		// Header spacing
		if ( $header_spacing ) {
			$header_spacing = str_replace( 'px', '', $header_spacing );
			kalium_append_custom_css( '.wrapper', "padding-top: {$header_spacing}px", '', true );
		}
	}
}

/**
 * Absolute header body class.
 *
 * @param array $classes
 *
 * @return array
 */
function _kalium_absolute_header_body_class( $classes ) {
	$queried_object_id = kalium_get_queried_object_id();
	$header_position   = kalium_header_get_option( 'position' );

	if ( 'absolute' === $header_position && ! post_password_required( $queried_object_id ) ) {
		$classes[] = 'header-absolute';
	}

	return $classes;
}

/**
 * Custom logo and menu options for current page.
 *
 * @return void
 */
function _kalium_header_custom_logo_and_menu_on_page() {
	if ( $queried_object_id = kalium_get_queried_object_id() ) {

		// Vars
		$custom_logo         = kalium_get_field( 'custom_logo', $queried_object_id );
		$custom_menu_skin    = kalium_get_field( 'custom_menu_skin', $queried_object_id );
		$sticky_menu_on_page = kalium_get_field( 'sticky_menu_on_page', $queried_object_id );
		$custom_sticky_logo  = kalium_get_field( 'custom_sticky_logo', $queried_object_id );
		$sticky_menu_skin    = kalium_get_field( 'sticky_menu_skin', $queried_object_id );

		$menu_skins = [
			'menu-skin-main',
			'menu-skin-dark',
			'menu-skin-light',
		];

		// Custom logo
		if ( $custom_logo && is_numeric( $custom_logo ) ) {
			$custom_logo_width = kalium_get_field( 'custom_logo_width' );

			add_filter( 'get_data_use_uploaded_logo', '__return_true' );
			add_filter( 'get_data_custom_logo_image', kalium_hook_return_value( $custom_logo ) );

			if ( is_numeric( $custom_logo_width ) && $custom_logo_width > 0 ) {
				add_filter( 'get_data_custom_logo_max_width', kalium_hook_return_value( $custom_logo_width ) );
			}
		}

		// Custom sticky logo
		if ( $custom_sticky_logo ) {
			add_filter( 'get_data_sticky_header_logo', kalium_hook_return_value( $custom_sticky_logo ) );
		}

		// Menu skin
		if ( $custom_menu_skin && in_array( $custom_menu_skin, $menu_skins ) ) {
			add_filter( 'get_data_menu_full_bg_skin', kalium_hook_return_value( $custom_menu_skin ) );
			add_filter( 'get_data_menu_standard_skin', kalium_hook_return_value( $custom_menu_skin ) );
			add_filter( 'get_data_menu_top_skin', kalium_hook_return_value( $custom_menu_skin ) );
			add_filter( 'get_data_menu_sidebar_skin', kalium_hook_return_value( $custom_menu_skin ) );
		}

		// Overwrite sticky header option for current page
		if ( in_array( $sticky_menu_on_page, [ 'enable', 'disable' ] ) ) {
			add_filter( 'get_data_sticky_header', ( $sticky_menu_on_page == 'enable' ? '__return_true' : '__return_false' ) );
		}

		// Custom sticky header skin
		if ( in_array( $sticky_menu_skin, $menu_skins ) ) {
			add_filter( 'get_data_sticky_header_skin', kalium_hook_return_value( $sticky_menu_skin ) );
		}
	}
}
