<?php
/**
 * Kalium WordPress Theme
 *
 * Header template functions.
 *
 * @author Laborator
 * @link   https://kaliumtheme.com
 */
if ( ! defined( 'ABSPATH' ) ) {
	exit; // Direct access not allowed.
}

/**
 * Display header template.
 */
if ( ! function_exists( 'kalium_header_display' ) ) {

    function kalium_header_display() {
		if ( ! kalium_show_header() ) {
			return;
		}

		// Load header template
		get_template_part( 'tpls/header-main' );
    }
}


/**
 * Display header content.
 */
if ( ! function_exists( 'kalium_header_content_display' ) ) {

	function kalium_header_content_display() {

		// Include custom header template
		kalium_get_template( 'custom-header.php' );
	}
}

/**
 * Top header bar.
 */
if ( ! function_exists( 'kalium_header_top_bar_display' ) ) {

	function kalium_header_top_bar_display() {

		// Check if enabled
		if ( ! kalium_header_has_top_bar() ) {
			return;
		}

		// Content entries
		$content_left  = kalium_parse_content_builder_field( kalium_get_theme_option( 'top_header_bar_content_left' ) );
		$content_right = kalium_parse_content_builder_field( kalium_get_theme_option( 'top_header_bar_content_right' ) );

		// Do not display if no content is available
		if ( empty( $content_left['entries'] ) && empty( $content_right['entries'] ) ) {
			return;
		}

		// Alignment
		$content_left_alignment  = kalium_get_array_key( $content_left['options'], 'alignment' );
		$content_right_alignment = kalium_get_array_key( $content_right['options'], 'alignment' );

		// Skin classes
		$container_skin = kalium_get_theme_option( 'top_header_bar_skin' );
		$entries_skin   = kalium_get_theme_option( 'top_header_bar_default_skin' );

		// Responsive visibility
		$show_on = [
			'desktop' => boolval( kalium_get_theme_option( 'top_header_bar_support_desktop', true ) ),
			'tablet'  => boolval( kalium_get_theme_option( 'top_header_bar_support_tablet', false ) ),
			'mobile'  => boolval( kalium_get_theme_option( 'top_header_bar_support_mobile', false ) ),
		];

		// Content args
		$content_args = [
			'default_skin' => $entries_skin,
			'menu_depth'   => 1,
		];

		// Container classes
		$classes = [
			'top-header-bar',
		];

		// Container skin
		if ( in_array( $container_skin, [ 'light', 'dark' ] ) ) {
			$classes[] = 'top-header-bar--skin-' . $container_skin;
		}

		// Responsive
		foreach ( $show_on as $device_type => $visible ) {
			if ( ! $visible ) {
				$classes[] = 'top-header-bar--hide-on-' . $device_type;
			}
		}

		// Row container classes
		$row_container_classes = [
			'top-header-bar--row-container',
		];

		// Assign container when fullwidth header is not present
		if ( false === kalium_is_fullwidth_header() ) {
			$row_container_classes[] = 'container';
		}

		// Left column class
		$left_column_classes = [
			'top-header-bar--column',
			'top-header-bar--column-content-left',
		];

		if ( $content_left_alignment ) {
			$left_column_classes[] = 'top-header-bar--column-alignment-' . $content_left_alignment;
		}

		// Left column class
		$right_column_classes = [
			'top-header-bar--column',
			'top-header-bar--column-content-right',
		];

		if ( $content_right_alignment ) {
			$right_column_classes[] = 'top-header-bar--column-alignment-' . $content_right_alignment;
		}

		// Custom background and border color
		$custom_background_color = kalium_get_theme_option( 'top_header_bar_background_color' );
		$custom_border_color     = kalium_get_theme_option( 'top_header_bar_border_color' );
		$custom_separator_color  = kalium_get_theme_option( 'top_header_bar_separator_color' );

		if ( $custom_background_color || $custom_border_color ) {
			kalium_append_custom_css( '.top-header-bar', [
				'background-color' => $custom_background_color,
				'border-bottom'    => $custom_border_color ? sprintf( '1px solid %s', $custom_border_color ) : '0px',
			] );
		}

		// Separator
		if ( $custom_separator_color ) {
			$classes[] = 'top-header-bar--with-sepearators';

			kalium_append_custom_css( '.top-header-bar--with-sepearators .header-block--item:before', [
				'background-color' => $custom_separator_color,
			] );
		}
		?>
        <div <?php kalium_class_attr( $classes ); ?>>

            <div <?php kalium_class_attr( $row_container_classes ); ?>>

                <div class="top-header-bar--row">

					<?php if ( ! empty( $content_left['entries'] ) ): ?>
                        <div <?php kalium_class_attr( $left_column_classes ); ?>>
							<?php
							// Left content entries
							kalium_header_render_menu_content_entries( $content_left, $content_args );
							?>
                        </div>
					<?php endif; ?>

					<?php if ( ! empty( $content_right['entries'] ) ): ?>
                        <div <?php kalium_class_attr( $right_column_classes ); ?>>
							<?php
							// Right content entries
							kalium_header_render_menu_content_entries( $content_right, $content_args );
							?>
                        </div>
					<?php endif; ?>

                </div>

            </div>

        </div>
		<?php
	}
}

/**
 * Render menu content entries.
 */
if ( ! function_exists( 'kalium_header_render_menu_content_entries' ) ) {

	function kalium_header_render_menu_content_entries( $content, $args = [] ) {
		$args = wp_parse_args( $args, [
			'default_skin' => kalium_get_theme_option( 'custom_header_default_skin' ),
			'menu_depth'   => 0,
		] );

		if ( is_array( $content ) && ! empty( $content['entries'] ) ) {
			$entries = $content['entries'];

			foreach ( $entries as $entry ) {
				$type    = $entry['content']['type'];
				$value   = $entry['content']['value'];
				$options = isset( $entry['options'] ) ? $entry['options'] : array();

				// Options by variable
				$option_skin  = kalium_get_array_key( $options, 'skin' );
				$option_align = kalium_get_array_key( $options, 'align' );
				$option_text  = kalium_get_array_key( $options, 'text' );

				// Inherit skin
				if ( 'inherit' === $option_skin ) {
					$option_skin = $args['default_skin'];
				}

				$item_classes = [
					'header-block--item',
				];

				// Responsive Hide on Device
				if ( isset( $options['hide_on'] ) ) {
					foreach ( $options['hide_on'] as $device => $enabled ) {
						if ( $enabled ) {
							$item_classes[] = 'header-block--item-hide-on-' . $device;
						}
					}
				}

				// Content HTML
				ob_start();

				// Handle content type
				switch ( $type ) {

					// Menu
					case 'menu':
						$option_submenu_dropdown_caret = kalium_get_array_key( $options, 'submenu_dropdown_caret' );

						// Classes
						$classes = [
							'standard-menu-container',
						];

						// Dropdown caret class
						if ( 1 !== $args['menu_depth'] && 'yes' === $option_submenu_dropdown_caret ) {
							$classes[] = 'dropdown-caret';
						}

						// Menu skin
						$classes[] = $option_skin;

						// Menu args
						$menu_args = [
							'depth' => $args['menu_depth'],
						];

						// Menu location or menu id to load
						if ( is_numeric( $value ) ) {
							$menu_args['menu'] = $value;
						} else if ( is_string( $value ) ) {
							$menu_args['theme_location'] = $value;
						}

						// Menu container
						kalium()->helpers->build_dom_element( [
							'tag_name'   => 'div',
							'attributes' => [
								'class' => kalium()->helpers->list_classes( $classes ),
							],
							'content'    => kalium_nav_menu( $menu_args ),
							'echo'       => true,
						] );
						break;

					// Elements
					case 'element':

						// Element types
						switch ( $value ) {

							// Raw text
							case 'raw-text':
								kalium()->helpers->build_dom_element( [
									'tag_name'   => 'div',
									'attributes' => [
										'class' => kalium()->helpers->list_classes( [
											'raw-text-widget',
											$option_skin,
										] ),
									],
									'content'    => wp_kses_post( do_shortcode( $option_text ) ),
									'echo'       => true,
								] );
								break;

							// Social networks
							case 'social-networks':
								$option_style             = kalium_get_array_key( $options, 'style' );
								$option_display_structure = kalium_get_array_key( $options, 'display_structure' );

								// Social networks args
								$social_networks_args = [
									'skin'            => $option_skin,
									'style'           => '',
									'include_icon'    => in_array( $option_display_structure, [
										'icon-title',
										'icon',
									] ),
									'include_title'   => in_array( $option_display_structure, [
										'icon-title',
										'title',
									] ),
									'rounded'         => false,
									'hover_underline' => false,
								];

								// Social networks style
								switch ( $option_style ) {

									// Rounded icons
									case 'rounded':
										$social_networks_args['rounded'] = true;
										break;

									// Rounded and colored icons
									case 'rounded-colored':
										$social_networks_args['rounded'] = true;
										$social_networks_args['style']   = 'color-background';
										break;

									// Rounded icons and colored on hover
									case 'rounded-colored-hover':
										$social_networks_args['rounded'] = true;
										$social_networks_args['style']   = 'color-background-hover';
										break;

									// Colored text
									case 'colored':
										$social_networks_args['style'] = 'color-text';
										break;

									// Colored text on hover
									case 'colored-hover':
										$social_networks_args['style'] = 'color-text-hover';
										break;

									// Colored background
									case 'colored-bg':
										$social_networks_args['style'] = 'color-background';
										break;

									// Colored background on hover
									case 'colored-bg-hover':
										$social_networks_args['style'] = 'color-background-hover';
										break;
								}

								// List social networks
								kalium_social_networks( $social_networks_args );
								break;

							// Toggle standard menu
							case 'open-standard-menu':
								$option_menu_id                = kalium_get_array_key( $options, 'menu_id' );
								$option_menu_position          = kalium_get_array_key( $options, 'menu_position' );
								$option_toggle_effect          = kalium_get_array_key( $options, 'toggle_effect' );
								$option_stagger_direction      = kalium_get_array_key( $options, 'stagger_direction' );
								$option_submenu_dropdown_caret = kalium_get_array_key( $options, 'submenu_dropdown_caret' );

								if ( preg_match( '/^menu-([0-9]+)$/', $option_menu_id, $matches ) ) {
									$option_menu_id = $matches[1];
								}

								$classes = [
									'standard-toggle-menu',
									'standard-toggle-menu--position-' . $option_menu_position,
								];

								// Wrapper start
								echo sprintf( '<div %1$s data-toggle-effect="%2$s" data-stagger-direction="%3$s">', kalium_class_attr( $classes, false ), $option_toggle_effect, $option_stagger_direction );

								// Menu
								$menu_classes = [
									'standard-toggle-menu--col',
									'standard-toggle-menu--menu',
									'standard-menu-container',
								];

								// Submenu dropdown caret
								if ( 'yes' === $option_submenu_dropdown_caret ) {
									$menu_classes[] = 'dropdown-caret';
								}

								if ( ! empty( $option_skin ) && 'inherit' !== $option_skin ) {
									$menu_classes[] = $option_skin;
								}

								echo sprintf( '<div %s>', kalium_class_attr( $menu_classes, false ) );
								echo kalium_nav_menu( $option_menu_id );
								echo '</div>';

								// Toggle menu button
								echo '<div class="standard-toggle-menu--col standard-toggle-menu--button">';
								kalium_header_menu_bars_button( 'standard-menu', $option_skin );
								echo '</div>';

								// Wrapper end
								echo '</div>';
								break;

							// Open top menu
							case 'open-top-menu':
								kalium_header_menu_bars_button( 'top-menu', $option_skin );
								break;

							// Open fullscreen menu
							case 'open-fullscreen-menu':
								$option_toggled_skin     = kalium_get_array_key( $options, 'toggled_skin' );
								$fullscreen_menu_options = [];

								if ( ! empty( $option_toggled_skin ) && 'inherit' !== $option_toggled_skin ) {
									$fullscreen_menu_options = array_merge( $fullscreen_menu_options, array(
										'skin_default' => $option_skin,
										'skin_active'  => $option_toggled_skin,
									) );
								}

								kalium_header_menu_bars_button( 'fullscreen-menu', $option_skin, $fullscreen_menu_options );

								add_action( 'kalium_header_content', 'kalium_header_fullscreen_menu', 30 );
								break;

							// Open sidebar menu
							case 'open-sidebar-menu':
								kalium_header_menu_bars_button( 'sidebar-menu', $option_skin );
								break;

							// Search field
							case 'search-field':
								kalium_header_search_field( $option_skin, 'right' === $option_align, true );
								break;

							// WPML language switcher
							case 'wpml-language-switcher':
								$option_text_format   = kalium_get_array_key( $options, 'display_text_format' );
								$option_flag_position = kalium_get_array_key( $options, 'flag_position' );
								$option_show_on       = kalium_get_array_key( $options, 'show_on' );
								$option_skip_missing  = kalium_get_array_key( $options, 'skip_missing' );

								$wpml_switcher_args = [
									'skin'          => $option_skin,
									'flag_position' => $option_flag_position,
									'display_text'  => $option_text_format,
									'show_on'       => $option_show_on,
									'skip_missing'  => is_array( $option_skip_missing ) && in_array( 'yes', $option_skip_missing ),
								];

								kalium_wpml_language_switcher( $wpml_switcher_args );
								break;

							// Mini cart icon
							case 'woocommerce-mini-cart':
								if ( function_exists( 'kalium_woocommerce_cart_menu_icon' ) ) {
									kalium_woocommerce_cart_menu_icon( $option_skin, $option_align );
								}
								break;

							// WooCommerce Cart Totals
							case 'woocommerce-cart-totals':
								$option_total_price = kalium_get_array_key( $options, 'total_price' );
								$option_text_before = kalium_get_array_key( $options, 'text_before' );
								$option_hide_empty  = 'yes' === kalium_get_array_key( $options, 'hide_empty' );

								kalium_woocommerce_cart_totals_widget( [
									'skin'        => $option_skin,
									'total_price' => $option_total_price,
									'text_before' => $option_text_before,
									'hide_empty'  => $option_hide_empty,
								] );
								break;
						}
						break;
				}

				// Element content
				if ( $element_content = ob_get_clean() ) {

					// Show DOM element
					kalium()->helpers->build_dom_element( [
						'tag_name'   => 'div',
						'attributes' => [
							'class' => kalium()->helpers->list_classes( $item_classes ),
						],
						'content'    => $element_content,
						'echo'       => true,
					] );
				}
			}
		}
	}
}

/**
 * Header left menu content (inline with the logo).
 */
if ( ! function_exists( 'kalium_header_content_left' ) ) {

	function kalium_header_content_left() {
		// Do not show if centered logo header is not selected
		if ( 'logo-centered' !== kalium_get_theme_option( 'custom_header_type' ) ) {
			return;
		}

		$menu_content = kalium_get_theme_option( 'custom_header_content_left' );
		$content      = kalium_parse_content_builder_field( $menu_content );

		$classes = [
			'header-block--column',
			'header-block--content-left',
			'header-block--auto-grow',
		];

		// Alignment
		if ( isset( $content['options']['alignment'] ) ) {
			$classes[] = 'header-block--align-' . $content['options']['alignment'];
		}

		?>
        <div <?php kalium_class_attr( $classes ); ?>>
			<?php
			// Render Content Entries
			kalium_header_render_menu_content_entries( $content );
			?>
        </div>
		<?php

	}
}

/**
 * Centered header right menu content (inline with the logo).
 */
if ( ! function_exists( 'kalium_header_content_right' ) ) {

	function kalium_header_content_right() {
		$menu_content = kalium_get_theme_option( 'custom_header_content_right' );
		$content      = kalium_parse_content_builder_field( $menu_content );

		$classes = [
			'header-block--column',
			'header-block--content-right',
			'header-block--auto-grow',
		];

		// Alignment
		if ( isset( $content['options']['alignment'] ) ) {
			$classes[] = 'header-block--align-' . $content['options']['alignment'];
		}
		?>
        <div <?php kalium_class_attr( $classes ); ?>>
			<?php
			// Render Content Entries
			kalium_header_render_menu_content_entries( $content );
			?>
        </div>
		<?php

	}
}

/**
 * Header content below the logo.
 */
if ( ! function_exists( 'kalium_header_content_below' ) ) {

	function kalium_header_content_below() {
		$menu_content = kalium_get_theme_option( 'custom_header_content' );
		$content      = kalium_parse_content_builder_field( $menu_content );

		// No entries found
		if ( empty( $content['entries'] ) ) {
			return;
		}

		$classes = [
			'header-block--column',
			'header-block--content-below',
			'header-block--auto-grow',
		];

		// Alignment
		if ( isset( $content['options']['alignment'] ) ) {
			$classes[] = 'header-block--align-' . $content['options']['alignment'];
		}

		?>
        <div class="header-block--row-container container">

            <div class="header-block--row header-block--row-secondary">

                <div <?php kalium_class_attr( $classes ); ?>>
					<?php
					// Render Content Entries
					kalium_header_render_menu_content_entries( $content );
					?>
                </div>

            </div>

        </div>
		<?php

	}
}

/**
 * Centered header logo.
 */
if ( ! function_exists( 'kalium_header_content_logo' ) ) {

	function kalium_header_content_logo() {

		?>
        <div class="header-block--column header-block--logo">
			<?php
			// Logo element
			kalium_logo_element();
			?>
        </div>
		<?php
	}
}

/**
 * Menu bars link.
 */
if ( ! function_exists( 'kalium_header_menu_bars_button' ) ) {

	function kalium_header_menu_bars_button( $action, $skin = '', $opts = [] ) {
		$opts = wp_parse_args( $opts, [
			'skin_default' => '',
			'skin_active'  => '',
		] );

		$classes = [
			'menu-bar',
		];

		if ( ! empty( $skin ) ) {
			$classes[] = $skin;
		}

		// Other attributes
		$atts = [
			'data-action="' . esc_attr( $action ) . '"',
		];

		if ( $opts['skin_default'] && $opts['skin_active'] ) {
			$atts[] = 'data-menu-skin-default="' . esc_attr( $opts['skin_default'] ) . '"';
			$atts[] = 'data-menu-skin-active="' . esc_attr( $opts['skin_active'] ) . '"';
		}

		// Trigger action link
		echo sprintf( '<a href="#" class="%s" %s>', kalium()->helpers->list_classes( $classes ), implode( ' ', $atts ) );

		// Icon or label
		kalium_menu_icon_or_label();

		echo '</a>';
	}
}

/**
 * Fullscreen Menu container.
 */
if ( ! function_exists( 'kalium_header_fullscreen_menu' ) ) {

	/**
	 * @param array $args {
	 *
	 * @type string $skin
	 * @type string $align
	 * @type bool   $footer_block
	 * @type bool   $translucent_background
	 * @type bool   $search_field
	 * @type bool   $submenu_dropdown_indicator
	 * @type array  $menu_args
	 * }
	 */
	function kalium_header_fullscreen_menu( $args = [] ) {
		if ( ! is_array( $args ) ) {
			$args = [];
		}

		$args = wp_parse_args( $args, [
			'skin'                       => kalium_get_theme_option( 'menu_full_bg_skin' ),
			'align'                      => kalium_get_theme_option( 'menu_full_bg_alignment' ),
			'footer_block'               => kalium_get_theme_option( 'menu_full_bg_footer_block' ),
			'translucent_background'     => kalium_get_theme_option( 'menu_full_bg_opacity' ),
			'search_field'               => kalium_get_theme_option( 'menu_full_bg_search_field' ),
			'submenu_dropdown_indicator' => kalium_get_theme_option( 'submenu_dropdown_indicator' ),
			'menu_args'                  => [
				'echo' => true,
			],
		] );

		// Full bg menu classes
		$classes = [
			'fullscreen-menu',
			'full-screen-menu',
			'menu-open-effect-fade',
		];

		if ( ! empty( $args['skin'] ) ) {
			$classes[] = $args['skin'];
		}

		if ( $args['submenu_dropdown_indicator'] ) {
			$classes[] = 'submenu-indicator';
		}

		if ( 'centered-horizontal' == $args['align'] ) {
			$classes[] = 'menu-horizontally-center';
		}

		if ( in_array( $args['align'], [ 'centered', 'centered-horizontal' ] ) ) {
			$classes[] = 'menu-aligned-center';
		}

		if ( $args['footer_block'] ) {
			$classes[] = 'has-fullmenu-footer';
		}

		if ( $args['translucent_background'] ) {
			$classes[] = 'translucent-background';
		}
		?>
        <div <?php kalium_class_attr( $classes ); ?>>
            <div class="fullscreen-menu-navigation">
                <div class="container">
                    <nav>
						<?php
						// Navigation
						echo kalium_nav_menu();

						// Search field
						if ( $args['search_field'] ) : ?>
                            <form class="search-form" method="get" action="<?php echo esc_url( home_url( '/' ) ); ?>" enctype="application/x-www-form-urlencoded">
                                <input id="full-bg-search-inp" type="search" class="search-field" value="<?php echo get_search_query(); ?>" name="s" autocomplete="off"/>
                                <label for="full-bg-search-inp">
									<?php
									// Search placeholder
									printf( '%s %s', esc_html__( 'Search', 'kalium' ), '<span><i></i><i></i><i></i></span>' );
									?>
                                </label>
                            </form>
						<?php endif; ?>
                    </nav>
                </div>
            </div>

			<?php if ( $args['footer_block'] ) : ?>
                <div class="fullscreen-menu-footer">
                    <div class="container">
                        <div class="right-part">
							<?php echo do_shortcode( '[lab_social_networks rounded]' ); ?>
                        </div>
                        <div class="left-part">
							<?php echo do_shortcode( kalium_get_theme_option( 'footer_text' ) ); ?>
                        </div>
                    </div>
                </div>
			<?php endif; ?>

        </div>
		<?php
	}
}

/**
 * Header search field.
 */
if ( ! function_exists( 'kalium_header_search_field' ) ) {

	/**
	 * @param string $skin
	 * @param bool   $align_right
	 * @param bool   $force_show
	 *
	 * @return void
	 */
	function kalium_header_search_field( $skin = '', $align_right = false, $force_show = false ) {
		if ( ! $force_show && ! kalium_get_theme_option( 'header_search_field' ) ) {
			return;
		}

		// Split text plugin is required
		kalium_enqueue( 'gsap-splittext-js' );

		// Classes
		$classes = [
			'header-search-input',
		];

		// Skin class
		if ( ! empty( $skin ) ) {
			$classes[] = $skin;
		}

		// Right aligned
		if ( $align_right ) {
			$classes[] = 'header-search-input--align-right';
		}

		$animation = kalium_get_theme_option( 'header_search_field_icon_animation' );
		?>
        <div <?php kalium_class_attr( $classes ); ?>>
            <form role="search" method="get" action="<?php echo esc_url( home_url() ); ?>">

                <div class="search-field">
                    <span><?php esc_html_e( 'Search site...', 'kalium' ); ?></span>
                    <input type="search" value="" autocomplete="off" name="s"/>
                </div>

                <div class="search-icon">
                    <a href="#" data-animation="<?php echo $animation; ?>">
						<?php echo kalium_get_svg_file( 'assets/images/icons/search.svg', null, [ 24, 24 ] ); ?>
                    </a>
                </div>
            </form>

        </div>
		<?php
	}
}

/**
 * Show Menu Bar (hambuger icon).
 */
if ( ! function_exists( 'kalium_menu_icon_or_label' ) ) {

	function kalium_menu_icon_or_label() {
		$menu_hamburger_custom_label = kalium_get_theme_option( 'menu_hamburger_custom_label' );

		if ( $menu_hamburger_custom_label ) {

			$label_show_text  = kalium_get_theme_option( 'menu_hamburger_custom_label_text' );
			$label_close_text = kalium_get_theme_option( 'menu_hamburger_custom_label_close_text' );
			$icon_position    = kalium_get_theme_option( 'menu_hamburger_custom_icon_position', 'left' );

			?>
            <span class="show-menu-text icon-<?php echo esc_attr( $icon_position ); ?>"><?php echo $label_show_text; ?></span>
            <span class="hide-menu-text"><?php echo $label_close_text; ?></span>

            <span class="ham"></span>
			<?php

		} else {
			?>
            <span class="ham"></span>
			<?php
		}
	}
}

/**
 * Display mobile menu.
 */
if ( ! function_exists( 'kalium_header_display_mobile_menu' ) ) {

	function kalium_header_display_mobile_menu() {
		if ( ! kalium_show_header() ) {
			return;
		}

		// Mobile menu
		get_template_part( 'tpls/menu-mobile' );
	}
}

/**
 * Display header top menu.
 */
if ( ! function_exists( 'kalium_header_display_top_menu' ) ) {

	function kalium_header_display_top_menu() {
		if ( ! kalium_show_header() ) {
			return;
		}

		// Menu type to use
		$main_menu_type = kalium_get_theme_option( 'main_menu_type' );

		// Top menu
		if ( 'top-menu' === $main_menu_type || kalium_get_theme_option( 'menu_top_force_include' ) || kalium_header_has_content_element_type( 'top-menu' ) ) {
			get_template_part( 'tpls/menu-top' );
		}
	}
}

/**
 * Display header side menu.
 */
if ( ! function_exists( 'kalium_header_display_side_menu' ) ) {

	function kalium_header_display_side_menu() {
		if ( ! kalium_show_header() ) {
			return;
		}

		// Menu type to use
		$main_menu_type = kalium_get_theme_option( 'main_menu_type' );

		// Sidebar menu
		if ( 'sidebar-menu' === $main_menu_type || kalium_get_theme_option( 'menu_sidebar_force_include' ) || kalium_header_has_content_element_type( 'sidebar-menu' ) ) {
			get_template_part( 'tpls/menu-sidebar' );
		}
	}
}
